
(sal/load-source! "sal-script-util.scm")

(define (prove-core assertion prove-proc display-ce-proc)
	(let* ((flat-module (slot-value assertion :module))
				 (_ (sal-module/display-var-info flat-module))
				 (fsm (sal-flat-module->sal-bdd-fsm flat-module)))
		(multiple-value-bind
				(valid? counter-example)
				(prove-proc assertion fsm)
			(cond 
			 (valid?
				(print "proved."))
			 ((eq? counter-example #unspecified)
				(print "invalid."))
			 (else
				(display-ce-proc counter-example))))))

(define (display-ce-default ce)
	(let ((counter-example (sal-derived-path->original-path ce)))
		(print-counter-example counter-example)))
	
(define (prove-invariant assertion)
	(prove-core assertion sal-module-models/smc-invariant-core display-ce-default))

(define (prove-ltl-liveness assertion)
	(prove-core assertion sal-module-models/smc-accepting display-ce-default))

(define (prove-ctl assertion)
	(prove-core assertion sal-module-models/smc-ctl-core (lambda (_) (print "invalid."))))

(front-end/set-categories! '("Help" "Misc" "Assertion" "Pretty Printing" "Code Transformations" "Abstraction" "Traceability" "Path Pretty Printing" "LTL Support" "BDD Interface"))

(gen-front-end sal-smc
							 "SAL Symbolic Model Checker"
"Usage: sal-smc [options] <context-name> <assertion-name>
   or  sal-smc [options] <file-name> <assertion-name>
   or  sal-smc [options] --assertion='<assertion-expr>'"
"Examples: 
  sal-smc peterson mutex

  sal-smc peterson invalid

  sal-smc ../tmp/peterson.sal invalid

  sal-smc ~/examples/peterson.sal mutex

  sal-smc --verbose=3 peterson mutex

  sal-smc -v 3 peterson mutex

  sal-smc -v 3 ~/examples/peterson.sal mutex

  sal-smc --assertion='peterson!mutex'

  sal-smc --assertion='(@ mutex peterson)'

  sal-smc --assertion='arbiter{;10}!at_most_one_ack'

  sal-smc --assertion='(@ at-most-one-ack (arbiter () (10)))'

  sal-smc --disable-traceability peterson invalid

  sal-smc --delta-path peterson invalid"
(lambda (else)
	(if *main-context-name*
		(if *main-assertion-name*
			(begin 
				(print-error "Illegal argument `" else "'. Usage:")
				(sal-smc/simple-help))
			(set! *main-assertion-name* else))
		(set! *main-context-name* else)))
(begin
	(check-assertion-name-ref sal-smc/simple-help)
	(sal/set-make-sal-string-reader-proc! make-lsal-string-reader)
	(unless *assertion*
		(load-context-if-file-name *main-context-name*))
	(let ((assertion-qualified-name (or *assertion*
																			(string-append "(@ " *main-assertion-name* " " (file-name->sal-context-name *main-context-name*) ")"))))
		(guess-qualified-name-parser! assertion-qualified-name)
		(let ((assertion (make-boolean-assertion assertion-qualified-name #t)))
			(check-if-supported-assertion assertion)
			(check-if-trivial-assertion assertion)
			(unless (instance-of? assertion <sal-module-models>)
				(print-error "Only simple assertions (MODULE-MODELS) are supported in the current version.")
				(exit -1))
			(cond
			 ((sal-module-models/invariant? assertion)
				(prove-invariant assertion))
			 ((sal-module-models/accepting? assertion)
				(prove-ltl-liveness assertion))
			 (else
				(print-error "You should use sal-wmc to check CTL formulas.")
				(exit -1)))))))

(sal-smc/main)
